﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LightningCommand.Actors
{
    class Meteorite
    {
        public Vector2 Position { get; set; }
        public Vector2 Velocity { get; set; }
        public bool IsKilled    { get; set; }
        public int targetIndex  { get; set; }

        private float speed = 40f;
        private Graphics.Particle.Engine particleEngine;
        

        public Meteorite(Vector2 position, Vector2 target)
        {
            Position = position;
            Velocity = Vector2.Normalize(target - position) * speed;
        }


        public void Initialize(ref ArtManager art) 
        {
            List<Texture2D> textures = new List<Texture2D>();
            textures.Add(art.textures["dot"]);
            textures.Add(art.textures["flare_sm"]);
            particleEngine = new Graphics.Particle.Engine(textures, Position, Graphics.Particle.Engine.ParticleType.trail);
            IsKilled = false;

        }


        public void Update(float deltaTime) 
        {
            if (Position.Y < 466)
            {
                Position += Velocity * deltaTime;
                particleEngine.EmitterLocation = Position;
            }
            
            int updateSpeed = 1;
            if (IsKilled)
                updateSpeed = 3; // accelerate trail removal
            particleEngine.Update(updateSpeed);
        }


        public void Draw(SpriteBatch spriteBatch, ref ArtManager art) 
        {
            // draw particle trail
            particleEngine.Draw(spriteBatch);
            // mark current head position
            Rectangle srcRect = new Rectangle(0, 0, art.textures["flare_sm"].Width, art.textures["flare_sm"].Height);
            Vector2 srcOrigin = new Vector2(art.textures["flare_sm"].Width / 2f, art.textures["flare_sm"].Height / 2f);
            spriteBatch.Draw(art.textures["flare_sm"], Position, srcRect, new Color(1.0f, 1.0f, 0.8f), 0, srcOrigin, 1.6f, SpriteEffects.None, 0);
        }

    }
}
