﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace LightningCommand
{
    enum eGameState
    {
        menu,
        reset,
        play,
        pause,
        highscore,
        credits
    }

    enum eMenuItem
    {
        begin,
        play,
        highscore,
        credits,
        quit,
        end
    }

    class GameStateManager
    {
        public eGameState state = eGameState.menu;
        public eMenuItem menuItem = eMenuItem.play;

        public bool isSoundClick;
        public bool isSoundActivate;
        public bool isSoundHowle;

        private bool isSelectUp = false;
        private bool isSelectDown = false;
        private bool isSelect = false;
        private bool isClicked = false;

        public GameStateManager()
        {
            isSoundClick = false;
            isSoundActivate = false;
            isSoundHowle = false;
        }


        public bool Update(Input input)
        {
            switch(state)
            {
                case eGameState.play:
                    if (input.escape && !isSelect) isSelect = true;
                    if(!input.escape && isSelect)
                    {
                        state = eGameState.menu;
                        isSelect = false;
                    }
                    if (input.rightClick == ButtonState.Pressed && !isClicked) isClicked = true;
                    if (input.rightClick == ButtonState.Released && isClicked)
                    {
                        state = eGameState.pause;
                        isClicked = false;
                    }

                    return true;

                case eGameState.pause:
                    if (input.rightClick == ButtonState.Pressed && !isClicked) isClicked = true;
                    if (input.rightClick == ButtonState.Released && isClicked)
                    {
                        state = eGameState.play;
                        isClicked = false;
                    }

                    /*if (input.escape && !isSelect) isSelect = true;
                    if(!input.escape && isSelect)
                    {
                        state = eGameState.play;
                        isSelect = false;
                    }*/
                    return true;

                case eGameState.menu:
                    if (input.upArrow && !isSelectUp) isSelectUp = true;
                    if(!input.upArrow && isSelectUp)
                    {
                        menuItem--;
                        isSelectUp = false;
                        isSoundClick = true;
                    }

                    if (input.downArrow && !isSelectDown) isSelectDown = true;
                    if(!input.downArrow && isSelectDown)
                    {
                        menuItem++;
                        isSelectDown = false;
                        isSoundClick = true;
                    }

                    // wrap around
                    if (menuItem == eMenuItem.begin) menuItem = eMenuItem.quit;
                    if (menuItem == eMenuItem.end) menuItem = eMenuItem.play;

                    if(input.select)
                    {
                        switch(menuItem)
                        {
                            case eMenuItem.play:
                                isSoundActivate = true;
                                state = eGameState.reset;// play; // was reset
                                break;
                            case eMenuItem.highscore:
                                isSoundHowle = true;
                                state = eGameState.highscore;
                                break;
                            case eMenuItem.credits:
                                isSoundHowle = true;
                                state = eGameState.credits;
                                break;
                            case eMenuItem.quit:
                                return false; // signal done
                        }
                    }
                    break;

                case eGameState.highscore:
                    if (input.escape) state = eGameState.menu;
                    break;

                case eGameState.credits:
                    if (input.escape) state = eGameState.menu;
                    break;
            }
            return true;
        }
    }

}
