﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Diagnostics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;

using LightningCommand.Graphics;



// check this out when moving to 3D
//
// https://github.com/RedpointGames/Protogame
//
//



namespace LightningCommand
{

    public enum eSound
    {
        click,
        activate,
        lightning,
        explosion,
        howle,
        ambient
    }


    public class LightningCommand : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        ArtManager art;
        Input input;
        GameStateManager gameState;

        IEnumerator<bool> coroutine = null;
        bool coroutineIsFinished = false;
        bool IsGameOver = false;
        bool IsScoringLevel = false;
        bool IsStartingNextLevel = false;

        List<Actors.Meteorite> metorites;

        List<WeaponFlash> weaponFlashes;
        bool isWeaponTriggered = false;

        List<Graphics.Particle.Engine> explosionEngine;
        
        List<Vector2> buildings;
        List<bool> buildingsActive;
        bool gunActive;

        List<SoundEffect> soundFX;

        LightningBolt lightning;
        Vector2 pos_targeting;
        int refreshCounter = 0;


        int Level = 1;
        int Score = 0;
        int NumCities = 6;
        float Ammo = 500;

        int MetoritesToClear;
        private Random random;

        private SpriteFont font_debug;
        private SpriteFont font_level;
        private SpriteFont font_score;

        private int AmmoScore = 0;
        private int CityScore = 0;

        private Screens.HighScore highScoreScreen;



        public LightningCommand()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            //this.IsFixedTimeStep = true;
            //this.graphics.SynchronizeWithVerticalRetrace = true;
            //this.TargetElapsedTime = new System.TimeSpan(0, 0, 0, 0, 33);
        }

        [DllImport("user32.dll", EntryPoint = "SetCursorPos")]
        private static extern bool SetCursorPos(int X, int Y);
        [DllImport("user32.dll", EntryPoint = "GetWindowRect")]
        private static extern bool GetWindowRect(IntPtr hwnd, out Rectangle rect);
        [DllImport("user32.dll", EntryPoint = "ClientToScreen")]
        private static extern bool ClientToScreen(IntPtr hwnd, ref Point pt);

        protected override void OnActivated(object sender, System.EventArgs args)
        { 
            Window.Title = "Active App";
            base.OnActivated(sender, args);
        }

        protected override void OnDeactivated(object sender, System.EventArgs args)
        {
            Window.Title = "Deactivated App";
            base.OnDeactivated(sender, args);
        }


        //  -------------------------------------------------------------------
        //                          Initialization
        //  -------------------------------------------------------------------
        protected override void Initialize()
        {
            input = new Input();
            gameState = new GameStateManager();
            random = new Random();
      
            
            pos_targeting = Vector2.Zero;
            /*
             texture = new Texture2D(this.GraphicsDevice, 100, 100);
            Color[] colorData = new Color[100*100];
            for (int i = 0; i < 10000; i++)
            {
                colorData[i] = Color.Red;
            }
            texture.SetData<Color>(colorData);
            */

            metorites = new List<Actors.Meteorite>();
            weaponFlashes = new List<WeaponFlash>();
            explosionEngine = new List<Graphics.Particle.Engine>();
            
            buildings = new List<Vector2>();
            buildings.Add(new Vector2(119, 375));
            buildings.Add(new Vector2(199, 375));
            buildings.Add(new Vector2(279, 375));
            buildings.Add(new Vector2(439, 375));
            buildings.Add(new Vector2(519, 375));
            buildings.Add(new Vector2(599, 375));
            buildingsActive = new List<bool>();
            buildingsActive.Add(true);
            buildingsActive.Add(true);
            buildingsActive.Add(true);
            buildingsActive.Add(true);
            buildingsActive.Add(true);
            buildingsActive.Add(true);
            gunActive = true;

            
            highScoreScreen = new Screens.HighScore();

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // load texture resources
            art = new ArtManager();
            string[] textureFileNames = { "targetingSight", "dot", "lightningCap", "lightningSeg", "flare_sm", "explosion", 
                                          "building_01", "building_02", "building_trim", "gun", 
                                          "sceneBkg_01", "UI/highscoreControls", "UI/menuControls", "UI/controlsControls",
                                          "controlsBkg", "menuBkg"};
            art.Initialize(this.Content, textureFileNames);

            font_debug = this.Content.Load<SpriteFont>("Fonts/Debug");
            font_level = this.Content.Load<SpriteFont>("Fonts/Level");
            font_score = this.Content.Load<SpriteFont>("Fonts/Score");

            soundFX = new List<SoundEffect>();
            soundFX.Add(this.Content.Load<SoundEffect>("Sounds/wav_click"));
            soundFX.Add(this.Content.Load<SoundEffect>("Sounds/wav_activate"));
            soundFX.Add(this.Content.Load<SoundEffect>("Sounds/wav_lightning"));
            soundFX.Add(this.Content.Load<SoundEffect>("Sounds/wav_explosion"));
            soundFX.Add(this.Content.Load<SoundEffect>("Sounds/wav_howle"));
            soundFX.Add(this.Content.Load<SoundEffect>("Sounds/wav_ambient"));

            // create base baddies and start their particle engines
            metorites.Add(new Actors.Meteorite(new Vector2(0, -10), Vector2.Zero));
            metorites[0].Initialize(ref art);
            metorites.Add(new Actors.Meteorite(new Vector2(0, -10), Vector2.Zero));
            metorites[1].Initialize(ref art);
            metorites.Add(new Actors.Meteorite(new Vector2(0, -10), Vector2.Zero));
            metorites[2].Initialize(ref art);;
            BeginLevel(Level);
        }

        protected override void UnloadContent()
        {
            art.Destroy();
        }





        //  ------------------------------------------------------------------------------------
        //                                   Game Loop
        //  ------------------------------------------------------------------------------------
        protected override void Update(GameTime gameTime)
        {
            Window.Title = "Lightning Command";
            if (IsActive)
            {
                float deltaTime = (float)gameTime.ElapsedGameTime.TotalSeconds;
                input.Update(deltaTime);

                if(!gameState.Update(input))
                {
                    highScoreScreen.WriteScores();
                    Exit();
                }
                if(gameState.state == eGameState.reset)
                {
                    // do reset
                    gameState.state = eGameState.play;
                    for (int i = 0; i < buildingsActive.Count; i++)
                        buildingsActive[i] = true;
                    Level = 0;
                    Score = 0;

                    explosionEngine.Clear();

                    metorites.Clear();
                    metorites.Add(new Actors.Meteorite(new Vector2(0, -10), Vector2.Zero));
                    metorites[0].Initialize(ref art);
                    metorites.Add(new Actors.Meteorite(new Vector2(0, -10), Vector2.Zero));
                    metorites[1].Initialize(ref art);
                    metorites.Add(new Actors.Meteorite(new Vector2(0, -10), Vector2.Zero));
                    metorites[2].Initialize(ref art); ;

                    BeginLevel(Level);
                }

                // menu sounds
                if (gameState.isSoundClick)
                {
                    soundFX[(int)eSound.click].Play();
                    gameState.isSoundClick = false;
                }
                if (gameState.isSoundActivate)
                {
                    soundFX[(int)eSound.howle].Play();
                    gameState.isSoundActivate = false;
                }
                if(gameState.isSoundHowle)
                {
                    soundFX[(int)eSound.howle].Play();
                    gameState.isSoundHowle = false;
                }


                if (gameState.state != eGameState.play)
                    return;

                
                
                
                





                // -------------------------------------------
                //                 game play
                // -------------------------------------------
                UpdatePlayerTargeting(deltaTime);
                UpdateEnemyBehavior(deltaTime);

                if (coroutine != null)
                {
                    coroutine.MoveNext();
                    if (coroutineIsFinished) coroutine = null;
                }

                IsGameOver = true;
                foreach(var building in buildingsActive)
                {
                    if (building)
                        IsGameOver = false;
                }
                if(IsGameOver)
                {
                    if (coroutineIsFinished)// || MetoritesToClear == 0)
                    {
                        highScoreScreen.SortNewEntry(Score);
                        gameState.state = eGameState.highscore;
                    }
                }


                /*
                GamePadCapabilities gamePadCaps = GamePad.GetCapabilities(PlayerIndex.One);
                if(gamePadCaps.IsConnected)
                {
                    GamePadState gamePadState = GamePad.GetState(PlayerIndex.One, GamePadDeadZone.None);
                    if(gamePadCaps.HasLeftXThumbStick)
                    {
                        if (gamePadState.ThumbSticks.Left.X < -0.1f)
                            position.X += gamePadState.ThumbSticks.Left.X * 100 * deltaTime;
                        if (gamePadState.ThumbSticks.Left.X > 0.1f)
                            position.X += gamePadState.ThumbSticks.Left.X * 100 * deltaTime;
                    }
                }
                */

                /*
                KeyboardState keyState = Keyboard.GetState();
                if (keyState.IsKeyDown(Keys.Escape))
                    Exit();

                System.Text.StringBuilder sb = new System.Text.StringBuilder();
                foreach(var key in keyState.GetPressedKeys())
                {
                    sb.Append("keys: ").Append(key).Append(" pressed ");
                }
                if (sb.Length > 0)
                    System.Diagnostics.Debug.WriteLine(sb.ToString());
                else
                    System.Diagnostics.Debug.WriteLine("No Keys pressed");

                if(keyState.IsKeyDown(Keys.Right) && !previousKeyState.IsKeyDown(Keys.Right))
                    position.X += 100 * deltaTime;
                if(keyState.IsKeyDown(Keys.Left) && !previousKeyState.IsKeyDown(Keys.Left))
                    position.X -= 100 * deltaTime;
                if(keyState.IsKeyDown(Keys.Up))
                    position.Y -= 100 * deltaTime;
                if(keyState.IsKeyDown(Keys.Down))
                    position.Y += 100 * deltaTime;

                // if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))

                previousKeyState = keyState;
                */

                base.Update(gameTime);
            }
        }


        private void UpdatePlayerTargeting(float deltaTime)
        {
            // player weapon targeting ------------------------------------------
            if (weaponFlashes.Count > 0)
            {
                if (!weaponFlashes[weaponFlashes.Count - 1].IsCharging)
                {
                    pos_targeting.X = input.cursorPos.X;
                    pos_targeting.Y = input.cursorPos.Y;
                }
            }
            else
            {
                pos_targeting.X = input.cursorPos.X;
                pos_targeting.Y = input.cursorPos.Y;
            }

            // player weapon firing --------------------------------------------
            if (input.leftClick == ButtonState.Pressed)
            {
                if (gunActive && Ammo > 0)
                {
                    if (isWeaponTriggered == false)
                    {
                        weaponFlashes.Add(new WeaponFlash(pos_targeting));
                        soundFX[(int)eSound.lightning].Play();
                    }
                    Ammo -= 1;
                    isWeaponTriggered = true;
                    if (refreshCounter == 0)
                        lightning = new LightningBolt(new Vector2(400, 440), pos_targeting);

                    refreshCounter++;
                    if (refreshCounter == 4)
                        refreshCounter = 0;
                }
            }

            if (lightning != null)
            {
                lightning.Update();
            }

            if (input.leftClick == ButtonState.Released)
            {
                if (isWeaponTriggered)
                {
                    isWeaponTriggered = false;
                    weaponFlashes[weaponFlashes.Count - 1].IsCharging = false;

                    /* // testing mouse cursor lock
                    Process[] processes = Process.GetProcessesByName("LightningCommand");
                    Process lc = processes[0];
                    IntPtr ptr = lc.MainWindowHandle;
                    Point pt = new Point();
                    ClientToScreen(ptr, ref pt);
                    SetCursorPos(pt.X + (int)pos_targeting.X, pt.Y + (int)pos_targeting.Y);
                    */
                }
            }
        }


        private void UpdateEnemyBehavior(float deltaTime)
        {
            foreach (var metorite in metorites)
            {
                metorite.Update(deltaTime);
                if (metorite.Position.Y > 465)
                {
                    // ground collision occurred, instantiate explosion
                    List<Texture2D> textures = new List<Texture2D>();
                    textures.Add(art.textures["dot"]);
                    explosionEngine.Add(new Graphics.Particle.Engine(textures, metorite.Position, Graphics.Particle.Engine.ParticleType.explosion));

                    soundFX[(int)eSound.explosion].Play();

                    metorite.Position = new Vector2(0, -100);
                    metorite.Velocity = Vector2.Zero;

                    if (metorite.targetIndex == 3)
                        gunActive = false;
                    else
                    {
                        int killIndex = metorite.targetIndex;
                        if (metorite.targetIndex > 3) killIndex--;
                        buildingsActive[killIndex] = false;
                    }

                    MetoritesToClear--;
                    if (MetoritesToClear == 0)
                    {
                        // check again for all building lost
                        bool isLiveBuilding = false;
                        for (int i = 0; i < buildingsActive.Count; i++)
                        {
                            if (buildingsActive[i])
                                isLiveBuilding = true;
                        }

                        if (isLiveBuilding)
                            coroutine = ScoreLevel();
                        else
                           coroutine = GameOver();

                        return;
                    }
                }
            }

            for (int flashIndex = 0; flashIndex < weaponFlashes.Count; flashIndex++)
            {
                weaponFlashes[flashIndex].Update(deltaTime);
                if (weaponFlashes[flashIndex].IsActive == false)
                {
                    weaponFlashes.RemoveAt(flashIndex);
                    flashIndex--;
                }
                else
                {
                    // do metorite collision
                    // scale 1.0f == 64 pixels (weapon) radius in game (32px * Scale) 
                    // scale 1.0f == 16 pixels (metorite) drawn at 1.6 scale == 25.6 diameter == 13px radius in game
                    foreach (var metorite in metorites)
                    {
                        Vector2 metoritePos = metorite.Position;
                        Vector2 weaponPos = weaponFlashes[flashIndex].Position;
                        float killZoneSquared = (32 * weaponFlashes[flashIndex].Scale) * (32 * weaponFlashes[flashIndex].Scale);
                        float distanceSquared = Math.Abs((metoritePos.X - weaponPos.X) * (metoritePos.X - weaponPos.X)) +
                                                Math.Abs((metoritePos.Y - weaponPos.Y) * (metoritePos.Y - weaponPos.Y));
                        if (distanceSquared < killZoneSquared)
                        {
                            // collided
                            metorite.Position = new Vector2(0f, -50f);
                            metorite.Velocity = Vector2.Zero;
                            metorite.IsKilled = true;
                            MetoritesToClear--;
                            Score += 25;
                            if (MetoritesToClear == 0 && !IsGameOver)
                            {
                                coroutine = ScoreLevel();
                                break;
                            }
                        }
                    }
                }
            }

            for (int explosionIndex = 0; explosionIndex < explosionEngine.Count; explosionIndex++)
            {
                explosionEngine[explosionIndex].Update(1);
                if (explosionEngine[explosionIndex].particles.Count == 0 && explosionEngine[explosionIndex].IsActive)
                {
                    explosionEngine.RemoveAt(explosionIndex);
                    explosionIndex--;
                }
            }
        }


        IEnumerator<bool> GameOver()
        {
            coroutineIsFinished = false;
            soundFX[(int)eSound.howle].Play();
            for (int i = 0; i < 420; i++) yield return false;

            coroutineIsFinished = true;
            yield return true;
        }


        IEnumerator<bool> ScoreLevel()
        {
            coroutineIsFinished = false;
            IsScoringLevel = true;
            int i = 0;
            while(i < Ammo)
            {
                i+=20;
                if(Ammo > 0) AmmoScore += 25;
                soundFX[(int)eSound.click].Play();
                yield return false;
                for (int wait = 0; wait < 5; wait++) yield return false;
            }

            i = 0;
            while(i<NumCities)
            {
                i++;
                CityScore += 100;
                soundFX[(int)eSound.click].Play();
                yield return false;
                for (int wait = 0; wait < 5; wait++) yield return false;
            }

            Score += AmmoScore + CityScore;

            for (int wait = 0; wait < 120; wait++) yield return false;
            
            IsScoringLevel = false;
            IsStartingNextLevel = true;
            for (int wait = 0; wait < 120; wait++) yield return false;


            coroutineIsFinished = true;
            AmmoScore = 0;
            CityScore = 0;
            Ammo = 500;
            gunActive = true;
            Level++;
            BeginLevel(Level);
            IsStartingNextLevel = false;
            yield return true;
        }



        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            // var fps = 1.0 / gameTime.ElapsedGameTime.TotalSeconds;
            //  Window.Title = fps.ToString();

            spriteBatch.Begin();
            if(gameState.state == eGameState.highscore)
            {
                spriteBatch.Draw(art.textures["controlsBkg"], Vector2.Zero, Color.White);
                spriteBatch.DrawString(font_level, "High Scores", new Vector2(290, 100), Color.White);
                spriteBatch.Draw(art.textures["UI/highscoreControls"], new Vector2(0, 0), Color.White);
                // call high scores draw
                highScoreScreen.Draw(spriteBatch, font_level, font_debug);
                spriteBatch.End();

                bool isButtonDown = true;

                if (!input.downArrow && !input.upArrow && !input.select && !input.leftArrow && !input.rightArrow)
                {
                    isButtonDown = false;
                    highScoreScreen.updateInterval = 0.5f; // all buttons up...be responsive on next press
                }

                highScoreScreen.updateInterval += 0.016f;// elapsedDeltaTime;
                if (highScoreScreen.updateInterval > 0.2f && isButtonDown)
                { // also allow button hold behavior to scroll
                    highScoreScreen.updateInterval = 0.0f;
                    highScoreScreen.Update(input);
                }
                return;
            }

            if(gameState.state == eGameState.credits)
            {
                spriteBatch.Draw(art.textures["controlsBkg"], Vector2.Zero, Color.White);
                spriteBatch.Draw(art.textures["UI/controlsControls"], new Vector2(0, 0), Color.White);
                spriteBatch.DrawString(font_level, "Controls", new Vector2(290, 120), Color.White);
                spriteBatch.DrawString(font_score, "Targeting", new Vector2(290, 190), Color.White);
                spriteBatch.DrawString(font_score, ": Mouse Move", new Vector2(440, 190), Color.White);
                spriteBatch.DrawString(font_score, "Fire Weapon", new Vector2(290, 220), Color.White);
                spriteBatch.DrawString(font_score, ": Left Mouse Click", new Vector2(440, 220), Color.White);
                spriteBatch.DrawString(font_score, "Increase Power", new Vector2(290, 250), Color.White);
                spriteBatch.DrawString(font_score, ": Left Mouse Hold", new Vector2(440, 250), Color.White);
                spriteBatch.DrawString(font_score, "Pause", new Vector2(290, 280), Color.White);
                spriteBatch.DrawString(font_score, ": Right Mouse Click", new Vector2(440, 280), Color.White);
                spriteBatch.End();
                return;
            }

            if(gameState.state == eGameState.menu)
            {
                spriteBatch.Draw(art.textures["menuBkg"], Vector2.Zero, Color.White);
                spriteBatch.Draw(art.textures["UI/menuControls"], new Vector2(0, 0), Color.White);
                Vector2 playPos = gameState.menuItem == eMenuItem.play ? new Vector2(415, 65) : new Vector2(435, 70);
                Vector2 scorePos = gameState.menuItem == eMenuItem.highscore ? new Vector2(360, 95) : new Vector2(405, 100);
                Vector2 creditPos = gameState.menuItem == eMenuItem.credits ? new Vector2(370, 125) : new Vector2(410, 130);
                Vector2 quitPos = gameState.menuItem == eMenuItem.quit ? new Vector2(422, 155) : new Vector2(437, 160);

                Color playColor = gameState.menuItem == eMenuItem.play ? Color.Yellow : Color.White;
                Color scoreColor = gameState.menuItem == eMenuItem.highscore ? Color.Yellow : Color.White;
                Color creditColor = gameState.menuItem == eMenuItem.credits ? Color.Yellow : Color.White;
                Color quitColor = gameState.menuItem == eMenuItem.quit ? Color.Yellow : Color.White;

                SpriteFont playFont = gameState.menuItem == eMenuItem.play ? font_level : font_score;
                SpriteFont scoreFont = gameState.menuItem == eMenuItem.highscore ? font_level : font_score;
                SpriteFont creditFont = gameState.menuItem == eMenuItem.credits ? font_level : font_score;
                SpriteFont quitFont = gameState.menuItem == eMenuItem.quit ? font_level : font_score;

                spriteBatch.DrawString(playFont, "PLAY", playPos, playColor);
                spriteBatch.DrawString(scoreFont, "HIGH SCORE", scorePos, scoreColor);
                spriteBatch.DrawString(creditFont, "CONTROLS", creditPos, creditColor);
                spriteBatch.DrawString(quitFont, "QUIT", quitPos, quitColor);
                spriteBatch.End();
                return;
            }



            spriteBatch.Draw(art.textures["sceneBkg_01"], Vector2.Zero, Color.White);
            spriteBatch.Draw(art.textures["building_trim"], new Vector2(20, 415), Color.White);

            int buildingIndex = 0;
            NumCities = 0;
            foreach (var buildingPos in buildings)
            {
                if (buildingsActive[buildingIndex])
                {
                    spriteBatch.Draw(art.textures["building_01"], buildingPos, Color.White);
                    NumCities++;
                }
                buildingIndex++;
            }
            if(gunActive)
                spriteBatch.Draw(art.textures["gun"], new Vector2(387, 410), Color.White);

            spriteBatch.Draw(art.textures["building_trim"], new Vector2(765, 415), Color.White);
            spriteBatch.End();


            spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.Additive);
            spriteBatch.Draw(art.textures["targetingSight"], pos_targeting - new Vector2(256, 128));
            spriteBatch.End();


            spriteBatch.Begin(SpriteSortMode.Texture, BlendState.Additive);
            
            if (lightning != null) lightning.Draw(spriteBatch, ref art);

            foreach (var metorite in metorites)
            {
                metorite.Draw(spriteBatch, ref art);
            }

            foreach (var flash in weaponFlashes)
            {
                flash.Draw(spriteBatch, ref art);
            }

            foreach(var explosion in explosionEngine)
            {
                explosion.Draw(spriteBatch);
            }


            spriteBatch.DrawString(font_score, "WAVE " + Level , new Vector2(50, 30), Color.White);
            spriteBatch.DrawString(font_score, " " + Score, new Vector2(350, 30), Color.White);
            spriteBatch.DrawString(font_score, "CITIES " + NumCities, new Vector2(700, 30), Color.White);
            spriteBatch.DrawString(font_debug, " " + Ammo, new Vector2(386, 465), Color.White);

            if(IsScoringLevel)
            {
                spriteBatch.DrawString(font_score, "BONUS POINTS", new Vector2(340, 200), Color.White);
                spriteBatch.DrawString(font_score, "AMMO", new Vector2(310, 230), Color.White);
                spriteBatch.DrawString(font_score, " " + AmmoScore, new Vector2(460, 230), Color.White);
                spriteBatch.DrawString(font_score, "CITIES", new Vector2(310, 260), Color.White);
                spriteBatch.DrawString(font_score, " " + CityScore, new Vector2(460, 260), Color.White);
            }

            if(IsStartingNextLevel)
            {
                spriteBatch.DrawString(font_score, "GET READY", new Vector2(360, 200), Color.White);
            }

            if(IsGameOver)
            {
                spriteBatch.DrawString(font_score, "GAME OVER", new Vector2(360, 200), Color.White);
            }

            if(gameState.state == eGameState.pause)
            {
                spriteBatch.DrawString(font_score, "PAUSED", new Vector2(360, 170), Color.White);
            }

            spriteBatch.End();
            base.Draw(gameTime);
        }


        private void BeginLevel(int level)
        {
            soundFX[(int)eSound.activate].Play();
            if ((level & 0x1) == 1)
            {
                // add another baddie on odd levels
                metorites.Add(new Actors.Meteorite(new Vector2(0, -10), Vector2.Zero));
                metorites[metorites.Count - 1].Initialize(ref art);
            }

            foreach(var metorite in metorites)
            {
                Vector2 startPos = new Vector2((int)(random.NextDouble()*800), -20 - (int)(random.NextDouble()*100));
                metorite.targetIndex = (int)(random.NextDouble() * 7);
                if (metorite.targetIndex == 7)
                    metorite.targetIndex = 6; // clamp to bounds...ignore the 1.0 random case
                Vector2 targetPos = new Vector2(160 + (metorite.targetIndex * 80), 465);

                metorite.Position = startPos;
                metorite.Velocity = Vector2.Normalize(targetPos - startPos) * 40f;
            }

            MetoritesToClear = metorites.Count;
        }
    }
}
