﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LightningCommand.Graphics
{
    class LightningBolt
    {
        public List<Line> Segments = new List<Line>();

        public float Alpha { get; set; }
        public float FadeOutRate { get; set; }
        public Color Tint { get; set; }

        public bool isComplete { get { return Alpha <= 0; } }


        public LightningBolt(Vector2 src, Vector2 dst) : this(src, dst, new Color(0.9f, 0.8f, 1.0f))
        {

        }

        public LightningBolt (Vector2 src, Vector2 dst, Color color)
        {
            Segments = CreateBolt(src, dst, 1.8f);
            Tint = color;
            Alpha = 1f;
            FadeOutRate = 0.03f;
        }


        public virtual void Update()
        {
            Alpha -= FadeOutRate;
        }


        public void Draw(SpriteBatch spriteBatch, ref ArtManager art)
        {
            if (Alpha <= 0)
                return;
            foreach(var segment in Segments)
            {
                segment.Draw(spriteBatch, Tint * (Alpha * 0.6f), ref art);
            }
        }

        static float Rand(Random random, double min, double max)
        {
            double range = max - min;// (double)float.MaxValue - (double)float.MinValue;
            double sample = random.NextDouble();
            return (float)((sample * range) + min);


            /* double mantissa = (random.NextDouble() * 2.0) - 1.0;
             double exponent = Math.Pow(2.0, random.Next(-126, 128));
             return (float)(mantissa * exponent);
             */
        }

        protected static List<Line> CreateBolt(Vector2 src, Vector2 dst, float thickness)
        {
            var results = new List<Line>();
            Vector2 tangent = dst - src;
            Vector2 normal = Vector2.Normalize(new Vector2(tangent.Y, -tangent.X));
            float length = tangent.Length();

            List<float> positions = new List<float>();
            positions.Add(0);

            Random autoRand = new Random();
            for (int i = 0; i < length / 4; i++)
                positions.Add(Rand(autoRand, 0, 1));
            positions.Sort();

            const float Sway = 80;
            const float Jaggedness = 1 / Sway;

            Vector2 prevPoint = src;
            float prevDisplacement = 0;
            for (int i = 1; i < positions.Count; i++)
            {
                float pos = positions[i];
                float scale = (length * Jaggedness) * (pos - positions[i - 1]);
                float envelope = pos > 0.95f ? 20 * (1 - pos) : 1;
                float displacement = Rand(autoRand, -Sway, Sway);
                displacement -= (displacement - prevDisplacement) * (1 - scale);
                displacement += envelope;

                Vector2 point = src + pos * tangent + displacement * normal;
                results.Add(new Line(prevPoint, point, thickness));
                prevPoint = point;
                prevDisplacement = displacement;
            }
            results.Add(new Line(prevPoint, dst, thickness));
            return results;
        }
    }
}
