﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LightningCommand.Graphics.Particle
{
    public class Engine
    {
        public enum ParticleType { trail, explosion };

        public Vector2 EmitterLocation { get; set; }
        public ParticleType Type { get; set; }

        public List<Texture2D> textures;
        public List<Particle> particles;

        public bool IsActive;
        private Random random;

        private int numExplosionParticles;
        private int maxExplosionParticles = 50;
        
        public Engine(List<Texture2D> textures, Vector2 location, ParticleType type)
        {
            EmitterLocation = location;
            Type = type;
            this.textures = textures;
            this.particles = new List<Particle>();
            random = new Random();
            numExplosionParticles = 0;
            IsActive = false;
        }

        private Particle GenerateNewTrailParticle()
        {
            Texture2D texture = textures[random.Next(textures.Count)];
            Vector2 position = EmitterLocation;
            Vector2 velocity = new Vector2(0f, 0f);
            float angle = 0f;
            float angularVelocity = 0.1f * (float)(random.NextDouble() * 2 - 1);
            Color color = new Color(0.9f, 0.7f, 0.0f);
            float size = (float)random.NextDouble() + 0.25f;
            int duration = 80 + random.Next(340);

            return new Particle(texture, position, velocity, angle, angularVelocity, color, size, duration);
        }

        private Particle GenerateNewExplosionParticle()
        {
            IsActive = true;
            numExplosionParticles++;
            Texture2D texture = textures[random.Next(textures.Count)];
            Vector2 position = EmitterLocation;
            Vector2 velocity = new Vector2(1f * (float)(random.NextDouble() * 2 - 1), -(float)(random.NextDouble() * 3));
            float angle = 0f;
            float angularVelocity = 0.1f * (float)(random.NextDouble() * 2 - 1);
            
            float green = (float)random.NextDouble();
            if (green > 0.7f)
                green = 0.7f;
            Color color = new Color(1f, green, 0.1f);
            float size = 1.0f;
            int duration = 240;

            return new Particle(texture, position, velocity, angle, angularVelocity, color, size, duration);

        }

        public void Update(int updateSpeed)
        {
            if ((float)random.NextDouble() > 0.8f)
            {
                int total = 1;
                if (Type == ParticleType.explosion)
                    total = 5;
                for (int i = 0; i < total; i++)
                {
                    switch(Type)
                    {
                        case ParticleType.trail:
                            particles.Add(GenerateNewTrailParticle());
                            break;
                        case ParticleType.explosion:
                            if (numExplosionParticles < maxExplosionParticles)
                            {
                                particles.Add(GenerateNewExplosionParticle());
                            }
                            break;
                    }
                }
            }
            else
            {
                switch(Type)
                {
                    case ParticleType.trail:
                        particles.Add(new Particle(textures[0], EmitterLocation, new Vector2(0f), 0f, 0f, new Color(0.4f, 0.4f, 0.1f), 0.9f, 700));
                        break;
                }
            }

            for(int particle=0; particle < particles.Count; particle++)
            {                
                particles[particle].Update(updateSpeed);
                switch(Type)
                {
                    case ParticleType.trail:
                        particles[particle].Size -= 0.002f;
                        break;
                    case ParticleType.explosion:
                        particles[particle].Size -= 0.008f;
                        particles[particle].Velocity += new Vector2(0.0f, 0.035f); // apply gravity
                        particles[particle].Velocity *= new Vector2(0.98f, 1.0f);  // apply horizontal dampening
                        if (particles[particle].Velocity.Y > 0.6f)
                            particles[particle].Velocity = new Vector2(particles[particle].Velocity.X, 0.6f); // terminal velocity
                        break;

                }

                if(particles[particle].Duration <= 0f ||
                   particles[particle].Size < 0f )
                {
                    particles.RemoveAt(particle);
                    particle--;
                }
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            for(int index = 0; index < particles.Count; index++)
            {
                particles[index].Draw(ref spriteBatch);
            }
        }


    }
}
