﻿using System;
using System.Collections.Generic;
using System.IO;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LightningCommand.Screens
{
    class HighScore
    {

        private List<string> names;
        private List<int> scores;

        private bool isAskingName = false;
        private int newEntryIndex = 0;
        private int maxScores = 4;

        private int alphaIndex = 0;
        private char[] alphas = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 
                                  'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '.' };
        private char initials0 = '.';
        private char initials1 = '.';
        private char initials2 = '.';
        private int initialIndex = 0;

        public float updateInterval = 0.0f;

        public HighScore()
        {
            names = new List<string>();
            scores = new List<int>();

            var fileStream = new FileStream("highScores.txt", FileMode.OpenOrCreate, FileAccess.Read);

            using (var streamReader = new StreamReader(fileStream, System.Text.Encoding.UTF8))
            {

                string line = streamReader.ReadLine();
                if (line != null)
                {
                    names.Add(line);
                    while ((line = streamReader.ReadLine()) != "")
                    {
                        names.Add(line);
                    }

                    while ((line = streamReader.ReadLine()) != null)
                    {
                        scores.Add(Convert.ToInt32(line));
                    }
                }
            }
        }


        public void WriteScores()
        {
            var fileStream = new FileStream("highScores.txt", FileMode.Create, FileAccess.Write);
            using (var streamWriter = new StreamWriter(fileStream, System.Text.Encoding.UTF8))
            {
                foreach (var name in names)
                {
                    if (name == "")
                        streamWriter.WriteLine("...");
                    else
                        streamWriter.WriteLine(name);
                }

                if (names.Count > 0)
                    streamWriter.WriteLine("");

                foreach (var score in scores)
                {
                    streamWriter.WriteLine(score);
                }
            }
        }


        public void SortNewEntry(int newScore)
        {
            int index = 0;
            foreach (var score in scores)
            {
                if (newScore > score)
                    break;
                index++;
            }

            if (index < scores.Count)
            {
                // do replacement
                int listEndIndex = scores.Count - 1;

                if (scores.Count < maxScores)
                {
                    names.Add(names[listEndIndex]);
                    scores.Add(scores[listEndIndex]);
                }

                while (index != listEndIndex)
                {
                    // shift lesser scores down the list
                    names[listEndIndex] = names[listEndIndex - 1];
                    scores[listEndIndex] = scores[listEndIndex - 1];
                    listEndIndex--;
                }

                names[index] = "";
                scores[index] = newScore;
                newEntryIndex = index;
                isAskingName = true;
                return;
            }

            if (index == scores.Count && scores.Count < maxScores)
            {
                // add to end of list
                newEntryIndex = index;
                isAskingName = true;
                names.Add("");
                scores.Add(newScore);
            }

            // list is full and this score wasn't good enough
        }


        public void Update(Input input)
        {
            if (isAskingName)
            {
                if (input.downArrow)
                    alphaIndex++;
                if (input.upArrow)
                    alphaIndex--;
                if (input.leftArrow)
                    initialIndex--;
                if (input.rightArrow)
                    initialIndex++;

                // wrap around
                if (alphaIndex < 0)
                    alphaIndex = 26;
                if (alphaIndex > 26)
                    alphaIndex = 0;
                if (initialIndex > 2)
                    initialIndex = 0;
                if (initialIndex < 0)
                    initialIndex = 2;

                // update user alpha scroll
                if (initialIndex == 0)
                    initials0 = alphas[alphaIndex];
                if (initialIndex == 1)
                    initials1 = alphas[alphaIndex];
                if (initialIndex == 2)
                    initials2 = alphas[alphaIndex];


                if (input.select && isAskingName)
                {
                    isAskingName = false;
                    names[newEntryIndex] = "" + initials0 + initials1 + initials2;
                    WriteScores();
                }
            }
        }

        public void Draw(SpriteBatch spritebatch, SpriteFont font, SpriteFont font_small)
        {
            Vector2 posS = new Vector2(270.0f, 140.0f);
            Vector2 posN = new Vector2(420.0f, 140.0f);
            Vector2 lineOffset = new Vector2(0.0f, 30.0f);

            for (int i = 0; i < names.Count; i++)
            {
                if (newEntryIndex == i && isAskingName)
                {
                    spritebatch.DrawString(font, "" + initials0 + initials1 + initials2, posS += lineOffset, Color.Yellow);
                }
                else
                    spritebatch.DrawString(font, names[i], posS += lineOffset, Color.White);

                spritebatch.DrawString(font, "" + scores[i], posN += lineOffset, Color.White);
            }
        }
    }
}

